@@ -10,27 +10,55 @@
 //! // An example image containing 2 RGB pixels.
 //! let mut pixels = vec![0, 1, 2, 3, 4, 5];
 //!
-//! let agcwd = agcwd::Agcwd::new(0.5);
+//! let agcwd = agcwd::Agcwd::new();
 //! agcwd.enhance_rgb_image(&mut pixels);
 //! ```
 #![warn(missing_docs)]
 
 mod color_format;
 
+/// [`Agcwd`] options.
+#[derive(Debug, Clone)]
+pub struct AgcwdOptions {
+    /// An algorithm parameter to adjust the shape of weighting distribution (WD).
+    ///
+    /// Defaults to `0.5`.
+    pub alpha: f32,
+
+    /// If `true`, the original image and the enhanced image by AGCWD are fused to generate the final image.
+    ///
+    /// Note that this is a this crate specific option (not defined by the AGCWD paper).
+    ///
+    /// Defaults to `false`.
+    pub fusion: bool,
+}
+
+impl Default for AgcwdOptions {
+    fn default() -> Self {
+        Self {
+            alpha: 0.5,
+            fusion: false,
+        }
+    }
+}
+
 /// [`Agcwd`] provides methods to enhance image contrast based on the [AGCWD] algorithm.
 ///
 /// [AGCWD]: https://ieeexplore.ieee.org/abstract/document/6336819/
-#[derive(Debug)]
+#[derive(Debug, Default)]
 pub struct Agcwd {
-    alpha: f32,
+    options: AgcwdOptions,
 }
 
 impl Agcwd {
-    /// Makes a new [`Agcwd`] instance.
-    ///
-    /// `alpha` is an algorithm parameter to adjust the shape of weighting distribution (WD).
-    pub fn new(alpha: f32) -> Self {
-        Self { alpha }
+    /// Makes a new [`Agcwd`] instance with the default options.
+    pub fn new() -> Self {
+        Self::with_options(Default::default())
+    }
+
+    /// Makes a new [`Agcwd`] instance with the given options.
+    pub fn with_options(options: AgcwdOptions) -> Self {
+        Self { options }
     }
 
     /// Enhances the contrast of an RGB image.
@@ -46,9 +74,9 @@ impl Agcwd {
     fn enhance_image<const N: usize>(&self, pixels: &mut [u8]) {
         let mut image = Image::<N>::new(pixels);
         let pdf = Pdf::new(&image);
-        let pdf_w = pdf.to_weighting_distribution(self.alpha);
+        let pdf_w = pdf.to_weighting_distribution(self.options.alpha);
         let cdf_w = Cdf::new(&pdf_w);
-        let curve = IntensityTransformationCurve::new(&cdf_w);
+        let curve = IntensityTransformationCurve::new(&cdf_w, self.options.fusion);
         image.update_pixels(|r, g, b| {
             let (h, s, v) = color_format::rgb_to_hsv(r, g, b);
             color_format::hsv_to_rgb(h, s, curve.0[usize::from(v)])
@@ -60,10 +88,12 @@ impl Agcwd {
 struct IntensityTransformationCurve([u8; 256]);
 
 impl IntensityTransformationCurve {
-    fn new(cdf: &Cdf) -> Self {
+    fn new(cdf: &Cdf, fusion: bool) -> Self {
         let mut curve = [0; 256];
         for (i, x) in cdf.0.iter().copied().enumerate() {
-            curve[i] = (255.0 * (i as f32 / 255.0).powf(1.0 - x)).round() as u8;
+            let v0 = i as f32;
+            let v1 = 255.0 * (v0 / 255.0).powf(1.0 - x);
+            curve[i] = if fusion { v0 * x + v1 * (1.0 - x) } else { v1 }.round() as u8;
         }
         Self(curve)
     }
@@ -164,14 +194,14 @@ mod tests {
     #[test]
     fn enhance_rgb_image_works() {
         let mut pixels = [1, 2, 3, 4, 5, 6];
-        let agcwd = Agcwd::new(0.5);
+        let agcwd = Agcwd::new();
         agcwd.enhance_rgb_image(&mut pixels);
     }
 
     #[test]
     fn enhance_rgba_image_works() {
         let mut pixels = [1, 2, 3, 4, 5, 6, 7, 8];
-        let agcwd = Agcwd::new(0.5);
+        let agcwd = Agcwd::new();
         agcwd.enhance_rgba_image(&mut pixels);
     }
 }
